/***************************************************************************
                          lmsensor.cpp  -  description
                             -------------------
    begin                : Mon Aug 6 2001
    copyright            : (C) 2001 by 
    email                : 
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "lmsensor.h"

#include "lmsensorschip.h"

#include "stdio.h"

LMSensor::LMSensor(SensorsList *parent): Sensor(parent)
{
  feature= -1;
}


LMSensor::~LMSensor(){
}


bool LMSensor::init(const sensors_feature_data **data, int *nr1,int *nr2)
{
 double min,max;
 const sensors_chip_name *chip_name= getChipName();
 const char* main_name = (*data)->name;
 feature= (*data)->number;
 char *label;
 QString str;

 bool min_max=false;
 while( (*data= sensors_get_all_features(*chip_name, nr1, nr2)) && (*data)->mapping!=SENSORS_NO_MAPPING) {
   str= (*data)->name;
   if(str.find("_min")>=0 || str.find("_low")>=0) {
     sensors_get_feature(*chip_name, (*data)->number, &valMin);
     min_max=true;
   } 
   else if(str.find("_max")>=0 || str.find("_over")>=0  || str.find("_high")>=0) {
     sensors_get_feature(*chip_name, (*data)->number, &valMax);
     min_max=true;
   }
 }

 double newVal;
 if ((sensors_get_ignored(*chip_name, feature) != 1) || (sensors_get_feature(*chip_name, feature, &newVal) != 0)) {
   return false;
 }

 if (strstr(main_name, "temp"))
 {
    setType(lmTemp);
    max= 65;
    min= 0;
 }
 else if (strstr(main_name, "fan"))
 {
    setType(lmFan);
    max= 10000;
    min= 3000;
 }
 else if (strstr(main_name, "alarm") || strstr(main_name, "sensor") || strstr(main_name, "vrm"))
 {
    return false;
 }
 else
 {
    setType(lmVoltage);
    max= 16;
    min= -16;
 }

 str.sprintf("%s.%s", chip_name->prefix, main_name);
 setName( str.latin1() );

 sensors_get_label(*chip_name,feature,&label);
 setDescription(QString(label));

 if(min_max){
   if(min>max) {
     double pivot= valMin;
     min= max;
     max= pivot;
   }
   setValueMax(max,dgCelsius);
   setValueMin(min,dgCelsius);
   setValue((max+min)/2,dgCelsius);
 }
 else {
   setValueMax(70,dgCelsius);
   setValueMin(0,dgCelsius);
   setValue(newVal,dgCelsius);
 }

 readConfig();
 updateValue();
 setValueIdeal(getValue());

 return true;
}

void LMSensor::updateValue()
{
 setValue( readSensorValue(), dgCelsius );
}

double LMSensor::readSensorValue()
{
 double newVal;
 const sensors_chip_name *chip_name= getChipName();
 sensors_get_feature(*chip_name, feature, &newVal);
 return newVal;
}

#define ABSOLUTE_VALUE(n)  ( (n)>=0 ? (n) : -(n) )
#define TRUNCATE_VALUE(n)  ( (double)(int)(n)    )

double LMSensor::calculateIdealValue()
{
 double value= readSensorValue();

 if(getType()==lmVoltage) {
     double decimals= 10;
     double module = ABSOLUTE_VALUE(value);
     if(module>3.0) {
       if( ABSOLUTE_VALUE( 12.0-value)<2.0 ) return  12.0;
       if( ABSOLUTE_VALUE(-12.0-value)<2.0 ) return -12.0;
       if( ABSOLUTE_VALUE(  5.0-value)<1.0 ) return   5.0;
       if( ABSOLUTE_VALUE( -5.0-value)<1.0 ) return  -5.0;
       if( ABSOLUTE_VALUE(  3.3-value)<0.3 ) return   3.3;
       if( ABSOLUTE_VALUE( -3.3-value)<0.3 ) return  -3.3;
       if(module>4.0) decimals= 1;
     }
     return TRUNCATE_VALUE((value * decimals)) / decimals;
  } else {
     return value;
  }
}

const sensors_chip_name *LMSensor::getChipName()
{
  return ((LMSensorsChip *)parent())->getChipName();
}
